﻿Type=Service
Version=3.82
B4A=true
@EndOfDesignText@
#Region  Service Attributes 
	#StartAtBoot: False
#End Region

Sub Process_Globals
	'These global variables will be declared once when the application starts.
	'These variables can be accessed from all modules.
	Dim Notification1 As Notification
	Dim MainSocket As Socket 	
	Dim SocStream As AsyncStreams 
	Dim LastRequest As String 	
	Dim MyTTS As TTS_Class 
End Sub

Sub Service_Create
	Notification1.Initialize
	Notification1.Icon = "icon"          'Mandatory! Without this line you won't see any notification
	Notification1.Vibrate = False
	Notification1.AutoCancel = True      'The notif will be canceled after user's click on it
	'Let's show a message or notification just to say the service has started
	MyTTS.Initialize 
	
	
End Sub

Sub GetLastAddress() As String 
	Dim m As Map 
	If File.Exists (File.DirInternal ,"mySettings.txt") Then
		m=File.ReadMap (File.DirInternal ,"mySettings.txt")
	End If
	Return m.GetDefault ("ServerAddr","192.168.0.55")	
End Sub

Sub Service_Start (StartingIntent As Intent)
	'Here we schedule the next run in 3 minutes. 
	Log("Service Start")
	StartServiceAt("", DateTime.Now + 3 * DateTime.TicksPerMinute, False)
	
	
	ShowUIMessage("Servizio attivato") 
	OpenMainSocket
End Sub


Sub Service_Destroy
	'The service has been destroyed (the user clicked on the 'kill Service' button)
	If MainSocket.Connected Then MainSocket.Close  
	If SocStream.IsInitialized Then SocStream.Close
	ShowUIMessage("service destroyed")
End Sub

Sub ShowUIMessage(msg As String )
	Main.LastMessage = msg 
	If Not(IsPaused(Main)) Then		
		CallSub(Main,"ShowLastMessage")
	Else
		CallSubDelayed(Main,"ShowLastMessage")
		'We use only 1 notification. It means that the next notification will overwrite this one
		Notification1.SetInfo("Socket Service", msg, Main)
		Notification1.Notify(1)			
	End If
	MyTTS.SpeakThis(msg)
End Sub

Sub SendData(msg As String)
	Dim Buffer() As Byte 
	If msg="" Then msg=LastRequest 	
		If msg<>"" Then
		Log("sendData=" & msg)		
		Buffer = msg.GetBytes("UTF8")
		SocStream.Write(Buffer)
	End If		
End Sub

Sub OpenMainSocket
	Dim addr As String 
	
	'main is the name of the module which contains the variable ServerAddr
	' The var ServerAddr has been declared in Process_Global, therefore is visible from all modules
	' But if the activity is finished or paused, then we cannot access the variable.
	If Not(IsPaused(Main)) Then addr = Main.ServerAddr
	If addr = "" Then addr = GetLastAddress
	
	MainSocket.Initialize("MainSocket")
	Log("Socket addr=" & addr)
	'MainSocket.Close  
	MainSocket.Connect(addr,9000,5000)
End Sub

Sub MainSocket_Connected(Connected As Boolean)As Boolean 
	Log("Socket connect=" & Connected)
	If Connected = True Then	
		'initialize the data stream and connect it to the socket
		'The eventName is used for the callback event function and normally is the same as the control
		'We use InitializePrefix instead of Initalize because we want the stream to take care of the 
		'data flush. With Prefix all 'messages' sent over the socket will have have a 4 bytes prefix
		'with the length of the message. The 'NewData' event will be fired only when the WHOLE message
		'has been received. It means that we don't have to deal with 'truncated' messages (received a first
		'part and then wait for other parts and glue the message's parts together...)
		'The Stream take care of this in a transparent way (we just receive the whole msg in the event)
		'BUT the Server part (written in VB6) MUST comply with this protocol... (se the VB6 sources)
		SocStream.InitializePrefix(MainSocket.InputStream, True, MainSocket.OutputStream,"SocStream")
		ShowUIMessage("Connected")
		
		SendData("GetAlarms")   'send a request to our server.
		'"GetAlarms" is just a string we sent to the server. The server will receive this command 
		'and will reply something...
	Else
		'show a notification. If the user click on the notification, the service Main is started
		ShowUIMessage("Unable to connect")
	End If

End Sub

Sub SocStream_NewData (Buffer() As Byte)
	'Received new data
    Dim msg As String
	Dim cMsg As String 
		
	Dim unsignedi As Int
	Dim signedb As Byte
	
    msg = BytesToString(Buffer, 0, Buffer.Length, "ASCII")
	'lvLog.AddSingleLine (msg)
	
	For i = 0 To msg.Length -1
		signedb = Buffer(i)
		unsignedi = Bit.AND(0xff, signedb)
		cMsg = cMsg & Chr(unsignedi)
	Next
	Log("Received " & cMsg)
	'Ok, new data received.
	'I'm using a Stream with the Prefix option. Therefore the message is whole and already clean (without prefix)
	'So we can process the message (see the message and do something related to the message...
	ProcessAnswer(cMsg)
		
End Sub

Sub SocStream_Terminated
	SocStream.Close 
	MainSocket.Close 
	'The server has terminated the connection.
	'The next scheduled service restart will (try to) reopen it.
End Sub

Sub ProcessAnswer(msg As String )
	'a new string has been received
	'now we need to process the string and see the information
	'In my protocol the string is made as follow:
	' command + "=" + data 
	' example: CUSTOMER 17=John Smith, john@smith.xxx, +389723897432 
	'     the data for CUSTOMER 17 are the response after a CUSTOMER 17 request
	
	Dim cmd As String, tmp As String , x As Int 
	
	x = msg.IndexOf("=")
	If x = -1 Then Return          'no data? just exit
	cmd = msg.SubString2(0,x)      'get the part before the '='
	tmp = msg.SubString (x+1)      'in tmp now I have the 'data' part
	Log("cmd=" & cmd)
	Log("Data=" & tmp)
	Select Case cmd.ToUpperCase 
		Case "ALARM"     		 'there is an alarm, show notification
			If tmp.ToUpperCase <>"NO ALARM" Then
				ShowUIMessage(tmp)
			End If
		Case Else
			ShowUIMessage("Command? " & cmd)			
	End Select
			
End Sub